Package: CNVfilteR
Type: Package
Title: Identifies false positives of CNV calling tools by using SNV
        calls
Version: 1.26.0
Authors@R: 
    c(person(
        given = "Jose Marcos",
        family = "Moreno-Cabrera",
        role = c("aut", "cre"),
        email = "jpuntomarcos@gmail.com",
        comment = c(ORCID = "0000-0001-8570-0345")
    ), person(
        given = "Bernat",
        family = "Gel",
        role = "aut"
    ))
Description: CNVfilteR identifies those CNVs that can be discarded by using the single nucleotide variant (SNV) calls that are usually obtained in common NGS pipelines.
License: Artistic-2.0
URL: https://github.com/jpuntomarcos/CNVfilteR
Encoding: UTF-8
RoxygenNote: 7.1.1
Depends: R (>= 4.3)
Imports: IRanges, GenomicRanges, SummarizedExperiment, pracma,
        regioneR, assertthat, karyoploteR, CopyNumberPlots, graphics,
        utils, VariantAnnotation, Rsamtools, GenomeInfoDb, Biostrings,
        methods
Suggests: knitr, BiocStyle, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Hsapiens.UCSC.hg19.masked, rmarkdown
biocViews: CopyNumberVariation, Sequencing, DNASeq, Visualization,
        DataImport
VignetteBuilder: knitr
BugReports: https://github.com/jpuntomarcos/CNVfilteR/issues
git_url: https://git.bioconductor.org/packages/CNVfilteR
git_branch: RELEASE_3_23
git_last_commit: 65126b1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:19:45 UTC; biocbuild
Author: Jose Marcos Moreno-Cabrera [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8570-0345>),
  Bernat Gel [aut]
Maintainer: Jose Marcos Moreno-Cabrera <jpuntomarcos@gmail.com>
